import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_boost/AIOService/NavigationService/service/NavigationService.dart';
import 'package:flutter_boost/container/boost_container.dart';
import 'package:flutter_boost/container/container_manager.dart';
import 'package:flutter_boost/messaging/page_result_mediator.dart';
import 'package:flutter_boost/router/router.dart';

import 'AIOService/loader/ServiceLoader.dart';
import 'container/container_coordinator.dart';
import 'observers_holders.dart';

typedef Widget PageBuilder(String pageName, Map params, String uniqueId);

class FlutterBoost {
  static final FlutterBoost _instance = FlutterBoost();
  final GlobalKey<ContainerManagerState> containerManagerKey =
      GlobalKey<ContainerManagerState>();

  final Router _router = Router();
  final ObserversHolder _observersHolder = ObserversHolder();
  final PageResultMediator _resultMediator = PageResultMediator();

  FlutterBoost() {
    ServiceLoader.load();
  }

  static FlutterBoost get singleton => _instance;

  static ContainerManagerState get containerManager =>
      _instance.containerManagerKey.currentState;

  static TransitionBuilder init([TransitionBuilder builder]) {
    return (BuildContext context, Widget child) {
      assert(child is Navigator, 'child must be Navigator, what is wrong?');

      //Logger.log('Running flutter boost opt!');

      final BoostContainerManager manager = BoostContainerManager(
          key: _instance.containerManagerKey, initNavigator: child);

      if (builder != null) {
        return builder(context, manager);
      } else {
        return manager;
      }
    };
  }

  ObserversHolder get observersHolder => _observersHolder;

  //Register a default page builder.
  void registerDefaultPageBuilder(PageBuilder builder) {
    ContainerCoordinator.singleton.registerDefaultPageBuilder(builder);
  }

  //Register page builder for a key.
  void registerPageBuilder(String pageName, PageBuilder builder) {
    ContainerCoordinator.singleton.registerPageBuilder(pageName, builder);
  }

  Future<bool> openPage(String name, Map params, bool animated) {
    return _router.openPage(name, params, animated);
  }

  Future<bool> closePage(
      String name, String pageId, Map params, bool animated) {
    return _router.closePage(name, pageId, params, animated);
  }

  Future<bool> closePageForContext(BuildContext context){
    BoostContainerSettings settings =  BoostContainer.of(context).settings;
    return closePage(settings.name, settings.uniqueId, settings.params, true);
  }

  Future<bool> openUrl(Map params) {
    return _router.openUrl(params);
  }

  Future<bool> openUrlForResult(Map params, PageResultHandler handler) {
    return _router.openUrlForResult(params, handler);
  }

  //Close currentPage page.
  Future<bool> closeCurPage(Map params) {
    return _router.closeCurPage(params);
  }

  //Should avoid this!
  //Close number of pages on top of native stack.
  void closePagesByCount(int count) {
    _router.closePagesByCount(count);
  }

  static void handleOnStarPage() async {
    final Map<dynamic, dynamic> pageInfo =
        await NavigationService.pageOnStart(<dynamic, dynamic>{});
    if (pageInfo == null || pageInfo.isEmpty) return;

    if (pageInfo.containsKey("name") &&
        pageInfo.containsKey("params") &&
        pageInfo.containsKey("uniqueId")) {
      ContainerCoordinator.singleton.nativeContainerDidShow(
          pageInfo["name"], pageInfo["params"], pageInfo["uniqueId"]);
    }
  }

  bool onPageResult(String key, Map<String, dynamic> resultData) {
    containerManager?.containerStateOf(key)?.performOnResult(resultData);
    _resultMediator.onPageResult(key, resultData);
    return true;
  }

  VoidCallback setPageResultHandler(String key, PageResultHandler handler) {
    return _resultMediator.setPageResultHandler(key, handler);
  }

  VoidCallback addContainerObserver(BoostContainerObserver observer) =>
      _observersHolder.addObserver<BoostContainerObserver>(observer);

  VoidCallback addBoostContainerLifeCycleObserver(
          BoostContainerLifeCycleObserver observer) =>
      _observersHolder.addObserver<BoostContainerLifeCycleObserver>(observer);

  VoidCallback addBoostNavigatorObserver(BoostNavigatorObserver observer) =>
      _observersHolder.addObserver<BoostNavigatorObserver>(observer);

  static const MethodChannel _channel = const MethodChannel('flutter_boost');
  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
}
